using System;
using System.IO;

using System.Runtime.Serialization;
using System.Xml.Serialization;

[XmlRoot("animal")]
public class Animal
{
	[XmlArray("eyes")]
	[XmlArrayItem("eye")]
	public Eye[] Eyes;
	[XmlElement("claws")]
	public Claw[] Claws;
	[XmlIgnore]
	public string SomeMember = "Some member";

	public Animal Friend;
}

public class Eye
{
	[XmlAttribute("vision")]
	public double Vision;

	public Eye()
	{
	}

	public Eye(double aVision)
	{
		Vision = aVision;
	}
}

public class Claw
{
	[XmlElement(ElementName="claw")]
	public string Description;

	public Claw()
	{
	}

	public Claw(string aDescription)
	{
		Description = aDescription;
	}
}

public class ControllingSerializationDemo
{
	public static void SerializeAnimalToXml(Animal aAnimal, string aFileName)
	{
		XmlSerializer xmlSerializer = new XmlSerializer(typeof(Animal));
		TextWriter writer = new StreamWriter(aFileName);
		using (writer)
		{
			xmlSerializer.Serialize(writer, aAnimal);
		}
	}

	public static Animal DeserializeAnimalFromXml(string aFileName)
	{
		TextReader reader = new StreamReader(aFileName);
		using (reader)
		{
			XmlSerializer xmlSer = new XmlSerializer(typeof(Animal));
			object deserializedAnimal = xmlSer.Deserialize(reader);
			return (Animal) deserializedAnimal;
		}
	}

	public static void Main()
	{
		Animal animal1 = new Animal();
		animal1.Eyes = new Eye[] {new Eye(1.05), new Eye(0.85)};
		animal1.Claws = new Claw[] {new Claw("Left claw"), new Claw("Right claw")};
	
		Animal animal2 = new Animal();
		animal2.Eyes = new Eye[] {new Eye(1.00), new Eye(1.00)};
		animal2.Claws = new Claw[] {new Claw("Beautiful claw")};
		
		animal1.Friend = animal2;
		// animal2.Friend = animal1;

		SerializeAnimalToXml(animal1, "animal.xml");
		Console.WriteLine("Animal serialized.");
		
		Animal deserializedAnimal = DeserializeAnimalFromXml("animal.xml");
		Console.WriteLine("Animal deserialized.");
	}
}